// modified by agraham 26/10/2010
// added CloseEventHack() method and a case "close" block to AddRunTimeEvent()
using System;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;

namespace Dbasic.EnhancedControls
{
	/// <summary>
	/// Summary description for CEnhancedForm.
	/// </summary>
	public class CEnhancedForm:Form,IEnhancedControl,IImage,IEnabled,IText
	{
		public Graphics graphics;
		public System.Windows.Forms.MainMenu mainMenu1;
		public string name;
		public Bitmap bitmap;
		public Rectangle destRect; 
		public bool activeState = false;
		public bool cancelClose = false;
		public bool hideEventAdded = false;
		public Bitmap foreBitmap;
		public Graphics foreGraphics;
		public bool foreLayer = false; 
		public ImageAttributes imageAttr = new ImageAttributes();
        private b4p b4p = null;
        private SolidBrush brush = new SolidBrush(Color.Black);
        private Pen pen = new Pen(Color.Black);
        private ImageAttributes backAttr = new ImageAttributes();
		public CEnhancedForm(b4p b4p)
		{
            this.b4p = b4p;
			InitializeComponent();
            this.MinimizeBox = false;

#if DESKTOP
            try
            {
            if (b4p.icon != "")
                this.Icon = new Icon(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(b4p.icon));
            }
            catch
            {}
            this.ClientSize = new System.Drawing.Size(b4p.screenX, b4p.screenY);
            //this.MaximumSize = new System.Drawing.Size(b4p.screenX + 7,
             //   b4p.screenY + 34);
            //this.MinimumSize = this.MaximumSize;
            this.MaximumSize = this.Size;
            this.MinimumSize = this.MaximumSize;
            this.Name = "CEnhancedForm";
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
            Other.SetScale(false, b4p.autoScale, this, b4p);
#endif
#if DEVICE
            Other.SetScale(true, b4p.autoScale, this, b4p);
#endif
            destRect = new Rectangle(0,0,this.ClientSize.Width,this.ClientSize.Height);
            bitmap = new Bitmap((int)(ClientSize.Width / b4p.fixX), (int)(ClientSize.Height / b4p.fixY));
			graphics = Graphics.FromImage(bitmap);
       }
       
#if DESKTOP
        public void show()
        {
            if (b4p.mainForm == null) //first form shown
                b4p.mainForm = this;
            b4p.shownForm = this;
            if (this.hideEventAdded == false)
            {
                this.Closing += new System.ComponentModel.CancelEventHandler(HideForm);
                this.hideEventAdded = true;
            }
            if (b4p.htSubs.Contains(this.name + "_show"))
                this.ShowEvent() ;
            if (b4p.alFormsDisplayOrder.Count > 0 &&
                        b4p.alFormsDisplayOrder[b4p.alFormsDisplayOrder.Count - 1] == this) //form.show twice
                return;
            this.Show();
            if (b4p.alFormsDisplayOrder.Count > 0)
                ((Form)b4p.alFormsDisplayOrder[b4p.alFormsDisplayOrder.Count - 1]).Hide();
            if (this == b4p.mainForm)
                b4p.alFormsDisplayOrder.Clear();
            else
                b4p.alFormsDisplayOrder.Remove(this);
            b4p.alFormsDisplayOrder.Add(this);
            this.activeState = true;

        }
        public void AddMenu()
        {
            if (this.mainMenu1.MenuItems.Count == 1)
			{
				this.MaximumSize = new Size(10000,10000);
				this.MinimumSize = new Size(0,0);
				this.ClientSize = new Size(b4p.screenX,b4p.screenY );
                this.MaximumSize = this.MinimumSize = this.Size;
			}
        }
        public void close()
        {
            if (this == b4p.mainForm || b4p.alFormsDisplayOrder.Contains(this))
				this.Close();
        }
#endif
#if DEVICE
        public void show()
        {
            if (b4p.mainForm == null) //first form shown
            {
                b4p.mainForm = this;
               // Graphics g = this.CreateGraphics();
               // b4p.scale = g.DpiX / 96;
            }
            b4p.shownForm = this;
            if (this.hideEventAdded == false)
            {
                this.Closing += new System.ComponentModel.CancelEventHandler(HideForm);
                this.hideEventAdded = true;
            }
            if (b4p.htSubs.Contains(this.name + "_show"))
                this.ShowEvent();
            this.Show();
            this.activeState = true;

        }
        public void close()
        {
            if (this == b4p.mainForm || this.Visible) //all forms that are shown are active
                this.Close();
        }
#endif
            

        void HideForm(object sender, System.ComponentModel.CancelEventArgs e)
        {
            if (this == b4p.mainForm)
            {
                if (!b4p.quitFlag)
                {
                    if (b4p.htSubs.Contains(b4p.mainForm.name + "_close"))
                        this.CloseEvent();
                    if (!b4p.quitFlag && b4p.mainForm.cancelClose)
                    {
                        e.Cancel = true;
                        return;
                    }
                }
                if (b4p.mainForm != null)
                    b4p.CloseProgram();
            }
            if (b4p.quitFlag) //never cancels the closing
                return;
            if (b4p.htSubs.Contains(this.name + "_close"))
                this.CloseEvent();
            if (this.cancelClose)
            {
                e.Cancel = true;
                return;
            }
            else //minimize instead of closing
            {
                e.Cancel = true;
                this.Hide();
#if DESKTOP
                b4p.alFormsDisplayOrder.Remove(this);
				((Form)b4p.alFormsDisplayOrder[b4p.alFormsDisplayOrder.Count-1]).Show();
#endif
            }
        }

		private void InitializeComponent()
		{
            this.mainMenu1 = new System.Windows.Forms.MainMenu();
            this.SuspendLayout();
            // 
            // CEnhancedForm
            // 
            this.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Inherit;
            this.ClientSize = new System.Drawing.Size(240, 266);
            this.Menu = this.mainMenu1;
            this.MinimizeBox = false;
            this.Name = "CEnhancedForm";
            this.ResumeLayout(false);

		}

        public void FixedInvalidate(Rectangle c)
        {
            if (b4p.fixX == 1 && b4p.fixY == 1)
            {
                this.Invalidate(c);
                return;
            }
            Rectangle rect = new Rectangle((int)(c.Left * b4p.fixX), (int)(c.Top * b4p.fixY)
                , (int)(c.Width * b4p.fixX), (int)(c.Height * b4p.fixY));
            this.Invalidate(rect);
        }
		protected override void OnPaint(PaintEventArgs e)
		{
            e.Graphics.DrawImage(bitmap, destRect, 0, 0, bitmap.Width, bitmap.Height,
                GraphicsUnit.Pixel, backAttr);
			if (foreLayer)
				e.Graphics.DrawImage(foreBitmap,destRect,0,0,bitmap.Width,bitmap.Height,
				GraphicsUnit.Pixel,imageAttr);
		}

		protected override void OnPaintBackground(PaintEventArgs pevent)
		{
			//empty
		}
		protected override void Dispose(bool disposing)
		{
			if (bitmap != null)
				bitmap.Dispose();
			if (foreBitmap != null)
				foreBitmap.Dispose();
			base.Dispose (disposing);
		}
		public string propName
		 {
			 get
			 {
				 return name;
			 }
		 }

		public Bitmap MyBitmap
		{
			get
			{
				return this.bitmap;
			}
            set
            {
                if (value == null)
                {
                    graphics.FillRectangle(new SolidBrush(BackColor), 0, 0,Width,Height);
                }
                else
                {
                    Rectangle srcRect = new Rectangle(0, 0, value.Width, value.Height);
                    graphics.DrawImage(value, new Rectangle(0,0, bitmap.Width, bitmap.Height), srcRect, GraphicsUnit.Pixel);
                }
                Refresh();
            }
		}


        b4p.del1 delKeyDown = null;
        b4p.del2 delMouseDown = null;
        b4p.del2 delMouseMove = null;
        b4p.del2 delMouseUp = null;
        b4p.del0 delClose = null;
        b4p.del0 delShow = null;
        private void KeyDownEvent(object sender, KeyEventArgs e)
        {
            b4p.sender = name;
            delKeyDown(((int)e.KeyCode).ToString(b4p.cul));
        }
        private void MouseDownEvent(object sender, System.Windows.Forms.MouseEventArgs e)
        {
            b4p.sender = name;
            delMouseDown((e.X/b4p.fixX).ToString(b4p.cul), (e.Y/b4p.fixY).ToString(b4p.cul));
        }
        private void MouseMoveEvent(object sender, System.Windows.Forms.MouseEventArgs e)
        {
            b4p.sender = name;
            delMouseMove((e.X / b4p.fixX).ToString(b4p.cul), (e.Y / b4p.fixY).ToString(b4p.cul));
        }
        private void MouseUpEvent(object sender, System.Windows.Forms.MouseEventArgs e)
        {
            b4p.sender = name;
            delMouseUp((e.X / b4p.fixX).ToString(b4p.cul), (e.Y / b4p.fixY).ToString(b4p.cul));
        }
        private void ShowEvent()
        {
            delShow = (b4p.del0)b4p.htSubs[name + "_show"];
            b4p.sender = name;
            delShow();
        }

        private void CloseEvent()
        {
            this.cancelClose = false;
            delClose = (b4p.del0)b4p.htSubs[name + "_close"];
            b4p.sender = name;
            delClose();
        }
// ADDED
        private void CloseEventHack(object sender, System.ComponentModel.CancelEventArgs e)
        {
            b4p.sender = name;
            delClose();
        }
//END ADDED

        public void AddEvents()
        {
            try
            {
                if (b4p.htSubs.Contains(this.name + "_mousedown"))
                {
                    delMouseDown = (b4p.del2)b4p.htSubs[name + "_mousedown"];
                    this.MouseDown += new System.Windows.Forms.MouseEventHandler(this.MouseDownEvent);
                }
                if (b4p.htSubs.Contains(this.name + "_mousemove"))
                {
                    delMouseMove = (b4p.del2)b4p.htSubs[name + "_mousemove"];
                    this.MouseMove += new System.Windows.Forms.MouseEventHandler(this.MouseMoveEvent);
                }
                if (b4p.htSubs.Contains(this.name + "_mouseup"))
                {
                    delMouseUp = (b4p.del2)b4p.htSubs[name + "_mouseup"];
                    this.MouseUp += new System.Windows.Forms.MouseEventHandler(this.MouseUpEvent);
                }
                if (b4p.htSubs.Contains(this.name + "_keypress"))
                {
                    delKeyDown = (b4p.del1)b4p.htSubs[name + "_keypress"];
                    this.KeyDown += new KeyEventHandler(KeyDownEvent);
                }
            }
            catch
            {
                throw new Exception("Error assigning event to " + name.Remove(0, 1) +
                    ".\nCheck the number of arguments of each event sub.");
            }
        }

        public void AddRunTimeEvent(string eventName,string subName)
        {
            try
            {
                switch (eventName)
                {
                    case "mousedown":
                        delMouseDown = (b4p.del2)b4p.htSubs[subName];
                        this.MouseDown += new System.Windows.Forms.MouseEventHandler(MouseDownEvent);
                        break;
                    case "mousemove":
                        delMouseMove = (b4p.del2)b4p.htSubs[subName];
                        this.MouseMove += new System.Windows.Forms.MouseEventHandler(this.MouseMoveEvent);
                        break;
                    case "mouseup":
                        delMouseUp = (b4p.del2)b4p.htSubs[subName];
                        this.MouseUp += new System.Windows.Forms.MouseEventHandler(this.MouseUpEvent);
                        break;
                    case "keypress":
                        delKeyDown = (b4p.del1)b4p.htSubs[subName];
                        this.KeyDown += new KeyEventHandler(this.KeyDownEvent);
                        break;
// ADDED 
                   case "close":
                        delClose = (b4p.del0)b4p.htSubs[subName];
                        this.Closing += new System.ComponentModel.CancelEventHandler(this.CloseEventHack);
                        break;
//END ADDED
                }
            }
            catch
            {
                throw new Exception("Error assigning event to " + name.Remove(0, 1) +
                    ".\nCheck the number of arguments of each event sub.");
            }
        }
        public override Color BackColor
        {
            get
            {
                return base.BackColor;
            }
            set
            {
                base.BackColor = value;
                this.graphics.FillRectangle(new SolidBrush(BackColor), 0, 0, Width, Height);
                this.Refresh();
            }
        }
       
        public void circle(Graphics g,int x1, int y1, int r, Color color, bool fill)
        {
            Rectangle rect = new Rectangle(x1 - r, y1 - r, 2 * r+1, 2 * r+1);
            if (fill) //optional F
            {
                brush.Color = color;
                g.FillEllipse(brush, x1 - r, y1 - r, 2 * r, 2 * r);
            }
            else
            {
                pen.Color = color;
                g.DrawEllipse(pen, x1 - r, y1 - r, 2 * r, 2 * r);
            }
            rect.Height += 1;
            rect.Width += 1;
            this.FixedInvalidate(rect);
        }
        public void line(Graphics g, int x1, int y1, int x2,int y2, Color color, bool box,bool fill)
        {
            Rectangle rect = new Rectangle(Math.Min(x1, x2), Math.Min(y1, y2),
                Math.Abs(x2 - x1), Math.Abs(y2 - y1));
            if (box)
            {
                if (!fill)
                {
                    pen.Color = color;
                    g.DrawRectangle(pen, rect);
                }
                else
                {
                    brush.Color = color;
                    g.FillRectangle(brush, rect);
                }
            }
            else
            {
                pen.Color = color;
                g.DrawLine(pen, x1, y1, x2, y2);
            }
            rect.Height += 1;
            rect.Width += 1;
            FixedInvalidate(rect);
        }
        public void drawString(Graphics g,  string s, double fontSize,int x1, int y1, int x2, int y2, Color col)
        {
            Rectangle rect = new Rectangle(Math.Min(x1, x2), Math.Min(y1, y2),
                    Math.Abs(x2 - x1), Math.Abs(y2 - y1));
            brush.Color = col;
            g.DrawString(s, new Font(this.Font.Name, (float)(fontSize / b4p.fixY), this.Font.Style),brush, (RectangleF)rect);
            rect.Height += 1;
            rect.Width += 1;
            this.FixedInvalidate(rect);
        }
        public void drawImage(Graphics g, Bitmap bitmap, int x1, int y1)
        {
            Rectangle rect = new Rectangle(x1,y1,bitmap.Width,bitmap.Height);
			g.DrawImage(bitmap,x1,y1,new Rectangle(0,0,bitmap.Width,bitmap.Height),GraphicsUnit.Pixel);
			rect.Height +=1;
			rect.Width +=1;
			FixedInvalidate(rect);
        }

        public void drawImage(Graphics g, Bitmap bitmap, int x1, int y1, int x2, int y2)
        {
            Rectangle rect = new Rectangle(Math.Min(x1, x2), Math.Min(y1, y2),
                       Math.Abs(x2 - x1), Math.Abs(y2 - y1));
            g.DrawImage(bitmap, rect,
                new Rectangle(0, 0, bitmap.Width, bitmap.Height), GraphicsUnit.Pixel);
            rect.Height += 1;
            rect.Width += 1;
            FixedInvalidate(rect);
        }
        public void polygon(Graphics g, double[,] xy, int start, int count, Color col, bool fill)
        {
            int xMax, yMax, xMin, yMin;
            Point[] p = new Point[count];
            xMin = xMax = p[0].X = (int)xy[start, 0];
            yMin = yMax = p[0].Y = (int)xy[start, 1];
            for (int i = 1; i < count; i++)
            {
                xMin = Math.Min(xMin, p[i].X = (int)xy[start + i, 0]);
                xMax = Math.Max(xMax, p[i].X);
                yMin = Math.Min(yMin, p[i].Y = (int)xy[start + i, 1]);
                yMax = Math.Max(yMax, p[i].Y);
            }
            FinishPolygon(g, p, xMin, yMin, xMax, yMax, fill, col);
        }
        public void polygon(Graphics g, double[] x, int startX, double[] y, int startY, int count, Color col
            , bool fill)
        {
            int xMax, yMax, xMin, yMin;
            Point[] p = new Point[count];
            xMin = xMax = p[0].X = (int)x[startX];
            yMin = yMax = p[0].Y = (int)y[startY];
            for (int i = 1; i < count; i++)
            {
                xMin = Math.Min(xMin, p[i].X = (int)x[startX + i]);
                xMax = Math.Max(xMax, p[i].X);
                yMin = Math.Min(yMin, p[i].Y = (int)y[startY + i]);
                yMax = Math.Max(yMax, p[i].Y);
            }
            FinishPolygon(g, p, xMin, yMin, xMax, yMax, fill, col);
        }
        public void polygon(Graphics g, string[,] xy, int start, int count,Color col, bool fill)
        {
            int xMax, yMax, xMin, yMin;
            Point[] p = new Point[count];
            xMin = xMax = p[0].X = (int)double.Parse(xy[start, 0], b4p.cul);
            yMin = yMax = p[0].Y = (int)double.Parse(xy[start, 1], b4p.cul);
            for (int i = 1; i < count; i++)
            {
                xMin = Math.Min(xMin, p[i].X = (int)double.Parse(xy[start + i, 0], b4p.cul));
                xMax = Math.Max(xMax, p[i].X);
                yMin = Math.Min(yMin, p[i].Y = (int)double.Parse(xy[start + i, 1], b4p.cul));
                yMax = Math.Max(yMax, p[i].Y);
            }
            FinishPolygon(g, p, xMin, yMin, xMax, yMax, fill, col);
        }
        public void polygon(Graphics g, string[] x, int startX, string[] y, int startY, int count, Color col
            , bool fill)
        {
            int xMax, yMax, xMin, yMin;
            Point[] p = new Point[count];
            xMin = xMax = p[0].X = (int)double.Parse(x[startX], b4p.cul);
            yMin = yMax = p[0].Y = (int)double.Parse(y[startY], b4p.cul);
            for (int i = 1; i < count; i++)
            {
                xMin = Math.Min(xMin, p[i].X = (int)double.Parse(x[startX + i], b4p.cul));
                xMax = Math.Max(xMax, p[i].X);
                yMin = Math.Min(yMin, p[i].Y = (int)double.Parse(y[startY + i], b4p.cul));
                yMax = Math.Max(yMax, p[i].Y);
            }
            FinishPolygon(g,p,xMin, yMin, xMax, yMax, fill, col);
        }
        private void FinishPolygon(Graphics g, Point[] p, int xMin, int yMin, int xMax, int yMax, bool fill, Color col)
        {
            Rectangle rect = new Rectangle(xMin, yMin, xMax - xMin + 1, yMax - yMin + 1);
            if (fill)
            {
                this.brush.Color = col;
                g.FillPolygon(brush, p);
            }
            else
            {
                this.pen.Color = col;
                g.DrawPolygon(pen, p);
            }
            FixedInvalidate(rect);
        }
        public bool ForeLayer
        {
            get
            {
                return foreLayer;
            }
            set
            {
                foreLayer = value;
                if (foreLayer && foreBitmap == null)
                {
                    foreBitmap = new Bitmap
                                ((int)(ClientSize.Width / b4p.fixX), (int)(ClientSize.Height / b4p.fixY));
                    foreGraphics = Graphics.FromImage(foreBitmap);
                    foreGraphics.FillRectangle(b4p.foreBrush, destRect);
                    imageAttr.SetColorKey(b4p.transColor, b4p.transColor);
                }
            }

        }



        string IEnhancedControl.propName
        {
            get { return this.name; }
        }


       
    }
}
